/**
*  Copyright (c) 2002 by Phil Hanna
*  All rights reserved.
*  
*  You may study, use, modify, and distribute this
*  software for any purpose provided that this
*  copyright notice appears in all copies.
*  
*  This software is provided without warranty
*  either expressed or implied.
*/
package com.jspcr.listeners.currency;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;

/**
* A servlet context listener that downloads currency exchange
* rates every 20 minutes, but only while this Web application is
* running.
*/
public class DownloadTimer implements ServletContextListener
{
   /**
   * Constant for twenty minutes (in milliseconds)
   */
   public static final long REFRESH_RATE = 20 * 60 * 1000;

   /**
   * Name of the rate cache attribute in the servlet context
   */
   public static final String CACHE_ATTRIBUTE =
      "com.jspcr.listeners.currency.rateCache";

   /**
   * The rate cache object
   */
   private RateCache cache;

   /**
   * Timer thread
   */
   private Thread timer;

   /**
   * The servlet context used for logging messages
   */
   private transient ServletContext context;

   // ===========================================
   //    These two methods implement the
   //    ServletContextListener interface
   // ===========================================

   /**
   * Called when the Web application is ready to service
   * requests.
   */
   public void contextInitialized(ServletContextEvent sce) 
   {
      context = sce.getServletContext();
      context.log("DownloadTimer: Starting download timer");

      // Create a rate cache that will maintain a
      // set of exchange rates.

      cache = new RateCache();

      // Store the rate cache in the servlet context

      context.setAttribute(CACHE_ATTRIBUTE, cache);

      // Schedule an update for every twenty minutes

      Runnable task = new Runnable() {
         public void run() {
            while (cache != null) {
               context.log("DownloadTimer: updating rates");
               cache.refresh();
               try {
                  Thread.sleep(REFRESH_RATE);
               }
               catch (InterruptedException e) {
                  break;
               }
            }
         }
      };

      // Start the timer

      timer = new Thread(task);
      timer.start();
   }

   /**
   * Called when the Web application is about to be shut down.
   */
   public void contextDestroyed(ServletContextEvent sce)
   {
      context = sce.getServletContext();
      context.log("DownloadTimer: Stopping download timer");

      // Remove the rate cache object from the servlet context

      context.removeAttribute(CACHE_ATTRIBUTE);

      // Stop the periodic download

      cache = null;
      timer.interrupt();
   }
}
