/**
*  Copyright (c) 2002 by Phil Hanna
*  All rights reserved.
*  
*  You may study, use, modify, and distribute this
*  software for any purpose provided that this
*  copyright notice appears in all copies.
*  
*  This software is provided without warranty
*  either expressed or implied.
*/
package com.jspcr.listeners.traffic;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.util.Date;
import javax.naming.*;
import javax.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
* A session listener that keeps track of the start time
* and duration of each session.  It also implements
* <code>ServletContextListener</code> so that it can
* start and stop the database connection it uses.
*/
public class Counter implements
   HttpSessionListener, ServletContextListener
{
   /**
   * Table of active sessions
   */
   private Map activeSessionMap = new HashMap();

   /**
   * The database connection
   */
   private Connection con;

   /**
   * The statement object used to update the database
   */
   private PreparedStatement stmt;

   // ===========================================
   //    These two methods implement the
   //    HttpSessionListener interface
   // ===========================================

   /**
   * Logs the start of a session
   */
   public void sessionCreated(HttpSessionEvent se)
   {
      HttpSession session = se.getSession();

      // Save the creation time in the active session map

      String id = session.getId();
      Date created = new Date();
      activeSessionMap.put(id, created);
   }

   /**
   * Logs the end of a session
   */
   public void sessionDestroyed(HttpSessionEvent se)
   {
      HttpSession session = se.getSession();

      // Retrieve the creation time from the active session map

      String id = session.getId();
      Date created = (Date) activeSessionMap.get(id);
      if (created == null) {
         // Ignore sessions whose start time we don't know
         return;
      }

      // Add a row to the session log table for this session

      Date destroyed = new Date();
      synchronized(this) {
         try {
            stmt.setString(1, id);
            stmt.setTimestamp(2, new Timestamp(created.getTime()));
            stmt.setTimestamp(3, new Timestamp(destroyed.getTime()));
            stmt.executeUpdate();
         }
         catch (SQLException e) {
            e.printStackTrace();
         }
      }
   }

   // ===========================================
   //    These two methods implement the
   //    ServletContextListener interface
   // ===========================================

   /**
   * Called when the Web application is ready to service
   * requests.
   */
   public void contextInitialized(ServletContextEvent sce) 
   {
      // Log the application start event

      ServletContext context = sce.getServletContext();
      context.log("Counter: Opening connection");

      try {

         // Get a database connection from a data source
         // specified in the deployment descriptor

         Context ctx = new InitialContext();
         DataSource ds = (DataSource) ctx.lookup
            ("java:comp/env/jdbc/SessionLog");
         con = ds.getConnection();

         // Precompile an SQL statement that will add
         // entries to the database

         String SQL = "INSERT INTO session_log VALUES(?,?,?)";
         stmt = con.prepareStatement(SQL);
      }
      catch (NamingException e) {
         e.printStackTrace();
      }
      catch (SQLException e) {
         e.printStackTrace();
      }
   }

   /**
   * Called when the Web application is about to be shut down.
   */
   public void contextDestroyed(ServletContextEvent sce)
   {
      // Log the application end event

      ServletContext context = sce.getServletContext();
      context.log("Counter: Closing connection");

      // Shut down the database objects

      if (stmt != null) {
         try { stmt.close(); }
         catch (SQLException e) {
            e.printStackTrace();
         }
      }
      if (con != null) {
         try { con.close(); }
         catch (SQLException e) {
            e.printStackTrace();
         }
      }
   }
}
