/**
*  Copyright (c) 2002 by Phil Hanna
*  All rights reserved.
*  
*  You may study, use, modify, and distribute this
*  software for any purpose provided that this
*  copyright notice appears in all copies.
*  
*  This software is provided without warranty
*  either expressed or implied.
*/
package com.jspcr.filters;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
* Filter that converts an HTTP GET request to an equivalent
* HTTP POST.
*/
public class PostFilter extends BaseFilter
{
   /**
   * Converts GET requests to POST requests.
   * @param request the original request object
   * @param response the original response object
   * @exception IOException if an I/O error occurs
   * @exception ServletException if an application error occurs.
   */
   public void doFilter(
         ServletRequest request,
         ServletResponse response,
         FilterChain chain)
      throws IOException, ServletException
   {
      // Cast the request so that we can get its
      // HTTP-specific attributes

      HttpServletRequest httpRequest =
         (HttpServletRequest) request;

      // If this is a GET request, create a custom
      // wrapper and substitute it for the actual
      // request

      if (httpRequest.getMethod().equalsIgnoreCase("GET")) {
         request = new PostWrapper(httpRequest);
      }

      // Pass the request on to the next processor in the
      // filter chain.

      chain.doFilter(request, response);
   }

   /**
   * Request wrapper that overrides the methods necessary 
   * to make the request look like a POST:
   *
   * <ul>
   *    <li>Adds a <code>content-type</code> header</li>
   *    <li>Adds a <code>content-length</code> header</li>
   *    <li>Adds a <code>cache-control</code> header</li>
   *    <li><code>getMethod()</code> returns POST</li>
   *    <li><code>getQueryString()</code> returns
   *     <code>null</code></li>
   * </ul>
   *
   * All other methods are automatically delegated to
   * the wrapped request.
   */
   public class PostWrapper
      extends HttpServletRequestWrapper
   {
      private String queryString;
      private InputStream queryStream;

      private ServletInputStream inputStream;
      private BufferedReader reader;
      private int contentLength;

      /**
      * Creates a new <code>PostWrapper</code>
      * for the specified request
      */
      public PostWrapper(HttpServletRequest request)
      {
         super(request);

         // Save the query string and note its length

         queryString = request.getQueryString();
         byte[] buffer = queryString.getBytes();
         queryStream = new ByteArrayInputStream(buffer);
         contentLength = buffer.length;
      }

      /**
      * Returns the specified header, which may be one of those
      * added to make this a POST request.
      * @param name the header name
      */
      public String getHeader(String name)
      {
         String header = null;

         if (name.equalsIgnoreCase("content-type"))
            header = "application/x-www-form-urlencoded";
         else if (name.equalsIgnoreCase("content-length"))
            header = String.valueOf(contentLength);
         else if (name.equalsIgnoreCase("cache-control"))
            header = "no-cache";
         else {
            HttpServletRequest request =
               (HttpServletRequest) getRequest();
            header = request.getHeader(name);
         }
         return header;
      }

      /**
      * Returns all the values of a particular header
      * @param name the header name
      */
      public Enumeration getHeaders(String name)
      {
         Enumeration enumHeaders = null;

         if (name.equalsIgnoreCase("content-type")) {
            Vector v = new Vector();
            v.addElement("application/x-www-form-urlencoded");
            enumHeaders = v.elements();
         }
         else if (name.equalsIgnoreCase("content-length")) {
            Vector v = new Vector();
            v.addElement(String.valueOf(contentLength));
            enumHeaders = v.elements();
         }
         else if (name.equalsIgnoreCase("cache-control")) {
            Vector v = new Vector();
            v.addElement("no-cache");
            enumHeaders = v.elements();
         }
         else {
            HttpServletRequest request =
               (HttpServletRequest) getRequest();
            enumHeaders = request.getHeaders(name);
         }
         return enumHeaders;
      }

      /**
      * Returns the names of all the headers in the request,
      * including the augmented ones.
      */
      public Enumeration getHeaderNames()
      {
         Vector vnames = new Vector();
         HttpServletRequest request =
            (HttpServletRequest) getRequest();
         Enumeration e = request.getHeaderNames();
         while (e != null && e.hasMoreElements()) {
            vnames.addElement(e.nextElement());
         }
         vnames.addElement("content-type");
         vnames.addElement("content-length");
         vnames.addElement("cache-control");
         return vnames.elements();
      }

      /**
      * Returns an input stream over the request body,
      * which actually comes from the query string.
      * @exception IOException if an I/O error occurs
      * @exception IllegalStateException if getReader()
      * was already called
      */
      public ServletInputStream getInputStream()
         throws IOException
      {
         if (inputStream != null)
            return inputStream;

         if (reader != null)
            throw new IllegalStateException
               ("getReader() was already called");

         inputStream = new ServletInputStream()
         {
            public int read() throws IOException
            {
               return queryStream.read();
            }
         };

         return inputStream;
      }

      /**
      * Returns a character reader over the request body.
      * @exception IOException if an I/O error occurs
      * @exception IllegalStateException if getInputStream()
      * was already called
      */
      public BufferedReader getReader()
         throws IOException
      {
         if (reader != null)
            return reader;

         if (inputStream != null)
            throw new IllegalStateException
               ("getInputStream() was already called");

         reader = new BufferedReader(
                  new StringReader(queryString));

         return reader;
      }

      /**
      * Returns the length of the query string
      */
      public int getContentLength()
      {
         return contentLength;
      }

      /**
      * Returns the <code>content-type</code> header.
      */
      public String getContentType()
      {
         return getHeader("content-type");
      }

      /**
      * Returns POST to indicate the HTTP method
      */
      public String getMethod()
      {
         return "POST";
      }

      /**
      * Suppresses the return of the real query string
      * so that this looks like a POST request
      */
      public String getQueryString()
      {
         return null;
      }
   }
}
