/**
*  Copyright (c) 2002 by Phil Hanna
*  All rights reserved.
*  
*  You may study, use, modify, and distribute this
*  software for any purpose provided that this
*  copyright notice appears in all copies.
*  
*  This software is provided without warranty
*  either expressed or implied.
*/
package com.jspcr.filters;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.sax.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
* Filter that applies an XSLT transformation to an
* XML response stream.
*/
public class XSLTFilter implements Filter
{
   /**
   * Saved reference to config object so that
   * we can get the servlet context later
   */
   private FilterConfig config;

   /**
   * A factory object that creates instances of the precompiled
   * XSLT stylesheet.
   */
   private Templates templates;

   /**
   * Called when the filter is placed into service.
   * @param config the filter configuration object
   * @exception ServletException if a fatal error occurs
   */
   public void init(FilterConfig config)
      throws ServletException
   {
      this.config = config;
      try {

         // Get the name of the XSLT stylesheet from
         // the init parameter in web.xml

         String xsltPath = config.getInitParameter("stylesheet");

         // Get a transformer source object for this
         // stylesheet

         ServletContext context = config.getServletContext();
         URL xsltURL = context.getResource(xsltPath);
         String xsltSystemId = xsltURL.toExternalForm();
         Source xsltSource = new StreamSource(xsltSystemId);

         // Precompile the stylesheet

         TransformerFactory tf = TransformerFactory.newInstance();
         templates = tf.newTemplates(xsltSource);
      }
      catch (MalformedURLException e) {
         throw new UnavailableException(e.getMessage());
      }
      catch (TransformerException e) {
         throw new UnavailableException(e.getMessage());
      }
   }

   /**
   * Releases the template factory
   */
   public void destroy()
   {
      templates = null;
   }

   /**
   * Applies an XSLT transformation to the response
   * @param request the original request object
   * @param response the original response object
   * @exception IOException if an I/O error occurs
   * @exception ServletException if an application error occurs.
   */
   public void doFilter(
         ServletRequest request,
         ServletResponse response,
         FilterChain chain)
      throws IOException, ServletException
   {
      // Create a servlet response wrapper that will
      // buffer the response data so that it can be
      // fed to the XSLT processor

      BufferedResponseWrapper wrappedResponse =
         new BufferedResponseWrapper
            (response, config.getServletContext());

      // Pass the request on to the next entry in
      // the filter chain

      chain.doFilter(request, wrappedResponse);

      // After the request is processed, retrieve
      // the output from the wrapper object

      String buffer = wrappedResponse.getBufferAsString();
      config.getServletContext().log("buffer=[" + buffer + "]");

      // Create a transformation source and result

      StringReader sr = new StringReader(buffer);
      Source source = new StreamSource(sr);

      StringWriter sw = new StringWriter();
      Result result = new StreamResult(sw);

      // Apply the XSLT stylesheet to create the HTML

      try {
         Transformer transformer = templates.newTransformer();
         transformer.transform(source, result);
      }
      catch (TransformerException e) {
         throw new ServletException(e.getMessage());
      }

      // Use the original response object to send
      // converted results back to the requesting client

      buffer = sw.toString();
      response.setContentType("text/html");
      response.setContentLength(buffer.length());
      PrintWriter out = response.getWriter();
      out.print(buffer);
      out.flush();
   }
}
