/**
*  Copyright (c) 2002 by Phil Hanna
*  All rights reserved.
*  
*  You may study, use, modify, and distribute this
*  software for any purpose provided that this
*  copyright notice appears in all copies.
*  
*  This software is provided without warranty
*  either expressed or implied.
*/
package com.jspcr.debug.tracer;

import java.io.*;
import java.net.*;
import java.util.*;

/**
* A proxy HTTP server that handles a single request
*/
public class RequestHandler extends Thread
{
   public static final String DEFAULT_HOST = "localhost";
   public static final int DEFAULT_PORT = 80;

   private Socket client;
   private Logger logger;
   private String host;
   private int port;

   // ===========================================
   //    Constructors
   // ===========================================

   /**
   * Creates a new <code>RequestHandler</code>
   * for the specified client
   */
   public RequestHandler(Socket client)
   {
      this.client = client;
   }

   // ===========================================
   //    Instance methods
   // ===========================================

   /**
   * Copies the request from the client to the server
   * and copies the response back to the client.
   */
   public void run()
   {
      try {

         // Open a socket to the web server

         if (host == null)
            host = DEFAULT_HOST;
         if (port <= 0)
            port = DEFAULT_PORT;

         Socket server = new Socket(host, port);

         // Open I/O streams to the client

         InputStream cin =
            new BufferedInputStream(client.getInputStream());
         OutputStream cout =
            new BufferedOutputStream(client.getOutputStream());

         // Open I/O streams to the server

         InputStream sin =
            new BufferedInputStream(server.getInputStream());
         OutputStream sout =
            new BufferedOutputStream(server.getOutputStream());
         
         // Copy request line and headers from client to server,
         // echoing to logger if specified.  Stop after the
         // first empty line (end of headers) 

         int contentLength = 0;
         StringBuffer sb = new StringBuffer();
         for (;;) {

            // Read a byte from client
            // and copy it to server

            int c = cin.read();
            sout.write(c);

            // Ignore CR at end of line

            if (c == '\r')
               continue;

            // If LF, process the line

            if (c == '\n') {
               String line = sb.toString();
               sb = new StringBuffer();

               // Log the line

               logger.log("C: " + line);

               // If this is an empty line,
               // there are no more headers

               if (line.length() == 0)
                  break;

               // If it is a content length header,
               // save the content length

               int p = line.indexOf(":");
               if (p != -1) {
                  String key = line.substring(0, p).trim();
                  String value = line.substring(p+1).trim();
                  if (key.equalsIgnoreCase("content-length"))
                     contentLength = Integer.parseInt(value);
               }
            }

            // Otherwise, append char to string buffer

            else
               sb.append((char) c);
         }
         sout.flush();

         // If content length was specified, read input stream
         // and copy to server

         if (contentLength > 0) {
            for (int i = 0; i < contentLength; i++) {
               int c = cin.read();
               sout.write(c);
            }
            sout.flush();
         }

         // Echo the response back to the client

         sb = new StringBuffer();
         while (true) {

            // Read a byte from server
            // and copy it to client

            int c = sin.read();
            cout.write(c);

            // Ignore CR at end of line

            if (c == '\r')
               continue;

            // If LF, process the line

            if (c == '\n') {
               String line = sb.toString();
               sb = new StringBuffer();

               // Log the line

               logger.log("S: " + line);

               // If this is an empty line,
               // there are no more headers

               if (line.length() == 0)
                  break;
            }

            // Otherwise, append char to string buffer

            else
               sb.append((char) c);
         }
         cout.flush();

         // Copy remaining bytes to client

         int bytesCopied = 0;
         while (true) {
            int c = sin.read();
            if (c == -1)
               break;
            cout.write(c);
            bytesCopied++;
         }
         if (bytesCopied > 0)
            cout.flush();

         // Close streams and sockets

         cin.close();
         cout.close();
         client.close();

         sin.close();
         sout.close();
         server.close();
      }
      catch (IOException e) {
         e.printStackTrace();
      }
   }

   // ===========================================
   //    Property setters
   // ===========================================

   /**
   * Sets the logger.
   * @param logger the logger.
   */
   public void setLogger(Logger logger)
   {
      this.logger = logger;
   }

   /**
   * Sets the host.
   * @param host the host.
   */
   public void setHost(String host)
   {
      this.host = host;
   }

   /**
   * Sets the port.
   * @param port the port.
   */
   public void setPort(int port)
   {
      this.port = port;
   }
}
